@@ -60,27 +60,6 @@ extEditorObserver.prototype = {
     }
 }
 
-//-----------------------------------------------------------------------------
-var strbundle;
-function getLocaleString(aName) {
-    try {
-        if (!strbundle) {
-            var strBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);
-            strbundle = strBundleService.createBundle("chrome://exteditor/locale/exteditor.properties");
-        }
-
-        if (strbundle)
-            return strbundle.GetStringFromName(aName);
-    }
-    catch (e) {
-        extEditorError("Cannot get the localized string bundle: " + e);
-    }
-
-    return null;
-}
-
-//-----------------------------------------------------------------------------
-//-----------------------------------------------------------------------------
 //-----------------------------------------------------------------------------
 
 var settingsObserver;
@@ -122,11 +101,11 @@ function launchExtEditor() {
         Recipients2CompFields(msgCompFields);
 
         if (prefEditHeaders[exteditor_SUBJECT]) content += exteditor_SUBJECT + ":   " + subject + newLine;
-        if (prefEditHeaders[exteditor_TO]) content += exteditor_TO + ":        " + msgCompFields.to + newLine;
-        if (prefEditHeaders[exteditor_CC]) content += exteditor_CC + ":        " + msgCompFields.cc + newLine;
-        if (prefEditHeaders[exteditor_BCC]) content += exteditor_BCC + ":       " + msgCompFields.bcc + newLine;
-        if (prefEditHeaders[exteditor_REPLY_TO]) content += exteditor_REPLY_TO + ":  " + msgCompFields.replyTo + newLine;
-        if (prefEditHeaders[exteditor_NEWSGROUP]) content += exteditor_NEWSGROUP + ": " + msgCompFields.newsgroups + newLine;
+        if (prefEditHeaders[exteditor_TO]) content += exteditor_TO + ":        " + normalizeRecipients(msgCompFields.to) + newLine;
+        if (prefEditHeaders[exteditor_CC]) content += exteditor_CC + ":        " + normalizeRecipients(msgCompFields.cc) + newLine;
+        if (prefEditHeaders[exteditor_BCC]) content += exteditor_BCC + ":       " + normalizeRecipients(msgCompFields.bcc) + newLine;
+        if (prefEditHeaders[exteditor_REPLY_TO]) content += exteditor_REPLY_TO + ":  " + normalizeRecipients(msgCompFields.replyTo) + newLine;
+        if (prefEditHeaders[exteditor_NEWSGROUP]) content += exteditor_NEWSGROUP + ": " + normalizeRecipients(msgCompFields.newsgroups) + newLine;
         content += headersEnd;
     }
 
@@ -144,9 +123,22 @@ function launchExtEditor() {
 
     var params = new Array(file);
     editorObserver = new extEditorObserver();
-    extEditorRunProgram(prefNotifierExe, params, editorObserver); // non blocking call
+    const launched = extEditorRunProgram(prefNotifierExe, params, editorObserver); // non blocking call
+
+    if (!launched) {
+        setEditorDisabled(false);
+    }
 }
 
+/**
+ * Normalizes an email address list.
+ *
+ * @param {string} fieldValue comma-separated list of email address that may be MIME escaped
+ * @returns {string} unescaped comma-separated list of email address
+ */
+function normalizeRecipients(fieldValue) {
+    return gMsgCompose.compFields.splitRecipients(fieldValue, false, {}).join(", ");
+}
 
 //-----------------------------------------------------------------------------
 function updateEditor() {
@@ -191,22 +183,29 @@ function updateEditor() {
                 }
 
                 var subject = headerHash[exteditor_SUBJECT.toLowerCase()];
-                if (subject !== undefined) {
-                    document.getElementById('msgSubject').value = subject;
-                    gMsgCompose.compFields.subject = subject;
+                if (subject !== undefined && subject !== GetMsgSubjectElement().value) {
+                    GetMsgSubjectElement().value = subject;
+                    SetComposeWindowTitle();
                 }
 
                 var msgCompFields = gMsgCompose.compFields;
 
                 Recipients2CompFields(msgCompFields);
 
-                if (prefEditHeaders[exteditor_TO]) msgCompFields.to = headerHash[exteditor_TO.toLowerCase()];
-                if (prefEditHeaders[exteditor_CC]) msgCompFields.cc = headerHash[exteditor_CC.toLowerCase()];
-                if (prefEditHeaders[exteditor_BCC]) msgCompFields.bcc = headerHash[exteditor_BCC.toLowerCase()];
-                if (prefEditHeaders[exteditor_REPLY_TO]) msgCompFields.replyTo = headerHash[exteditor_REPLY_TO.toLowerCase()];
-                if (prefEditHeaders[exteditor_NEWSGROUP]) msgCompFields.newsgroups = headerHash[exteditor_NEWSGROUP.toLowerCase()];
+                let knownTypes = new Set();
+                exteditorEachAddressType((recipientType, compFieldKey, headerKey) => {
+                    knownTypes.add(recipientType);
+                })
 
-                CompFields2Recipients(msgCompFields);
+                exteditorClearRecipientOfType(knownTypes);
+                awCleanupRows();
+
+                exteditorEachAddressType((recipientType, compFieldKey, headerKey) => {
+                    let addresses = headerKey ? headerHash[headerKey] : msgCompFields[compFieldKey];
+                    awAddRecipients(msgCompFields, recipientType, addresses);
+                });
+
+                awCleanupRows();
             }
         } else {
             // No headers edition here
@@ -218,6 +217,7 @@ function updateEditor() {
         }
     } catch (e) {
         // A message could be displayed her, but I don't wan't to bother with localizations...
+        console.log("exteditor.updateEditor()", e)
     } finally {
         setEditorDisabled(false);
     }
@@ -231,7 +231,7 @@ function updateEditor() {
         // Don't use rebuildDocumentFromSource() here: it turns the editor in a
         // html mode in  which multiple spaces disapear.
         var wholeDocRange = editor.document.createRange();
-        var rootNode = editor.rootElement.QueryInterface(Components.interfaces.nsIDOMNode);
+        var rootNode = editor.rootElement;
         wholeDocRange.selectNodeContents(rootNode);
         editor.selection.addRange(wholeDocRange);
         try {
@@ -239,19 +239,39 @@ function updateEditor() {
         } catch (e) {
             // The selection did not exist yet. Everything should be fine
         }
-        editor.QueryInterface(Components.interfaces.nsIEditorMailSupport).insertTextWithQuotations(messageText);
+        if (messageText) {
+            editor.insertText(messageText);
+        }
     }
 }
 
-//-----------------------------------------------------------------------------
-function extEditorError(msg) {
-    msg = "ExtEditor: " + msg;
-    alert(msg);
+/**
+ * Enumerates known address field types along with corresponding keys.
+ *
+ * @param cb {(recipientType: string, compFieldKey: string, headerKey?: string) => void}
+ */
+function exteditorEachAddressType(cb) {
+    cb("addr_reply", "replyTo", prefEditHeaders[exteditor_REPLY_TO] ? exteditor_REPLY_TO.toLowerCase() : undefined);
+    cb("addr_to", "to", prefEditHeaders[exteditor_TO] ? exteditor_TO.toLowerCase() : undefined);
+    cb("addr_cc", "cc", prefEditHeaders[exteditor_CC] ? exteditor_CC.toLowerCase() : undefined);
+    cb("addr_bcc", "bcc", prefEditHeaders[exteditor_BCC] ? exteditor_BCC.toLowerCase() : undefined);
+    cb("addr_newsgroups", "newsgroups", prefEditHeaders[exteditor_NEWSGROUP] ? exteditor_NEWSGROUP.toLowerCase() : undefined);
 }
 
-//-----------------------------------------------------------------------------
-//-----------------------------------------------------------------------------
-//-----------------------------------------------------------------------------
+/**
+ * Clears address fields which have specified types.
+ *
+ * @param {Set<string>} recipientTypeSet
+ */
+function exteditorClearRecipientOfType(recipientTypeSet) {
+    for (let row = 1, n = awGetNumberOfRecipients(); row <= n; row++) {
+        let popup = awGetPopupElement(row);
+        if (recipientTypeSet.has(popup.value)) {
+            let input = awGetInputElement(row);
+            awSetInputAndPopupValue(input, "", popup, "addr_to", -1);
+        }
+    }
+}
 
 //-----------------------------------------------------------------------------
 function tryCloseExtEditor() {
@@ -422,11 +442,15 @@ function initButton(editorExe, editHtmlAsHtml) {
 
 //-----------------------------------------------------------------------------
 function isEditAsHtml() {
-    // why is this checked attribute returned as a string rather than as a bool ???
-    return (
-        IsHTMLEditor() &&
-        document.getElementById('exteditor_editAsHtml').getAttribute('checked') == "true"
-    );
+    if (!IsHTMLEditor()) { return false; }
+
+    var asHtml = document.getElementById('exteditor_editAsHtml');
+    if (!asHtml) {
+        // The button is not in the palette bar, check for preference
+        return nsPreferences.getBoolPref('exteditor.html.editAsHtml', true)
+    }
+
+    return asHtml.getAttribute('checked') == "true";
 }
 
 //-----------------------------------------------------------------------------
